/************************************************************************/
/*																		*/
/*	main.c	--	Main program file for the PmodRF1 reference design		*/
/*				Base Temperature Station								*/
/*																		*/
/************************************************************************/
/*	Author: 	Mark Taylor												*/
/*	Copyright	2009, Digilent Inc.										*/
/************************************************************************/
/*  Module Description: 												*/
/*	This program is the base unit of a home temperature station.  The	*/
/*	remote unit sleeps for 30 seconds and wakes to take a temp reading	*/
/*	along with a battery ready and sends us the results via a PmodRF1.	*/
/*																		*/
/*	This unit receives those packets and displays that information on a */
/*	PmodCLS along with an base temperature reading taken from a 		*/
/*	PmodTMP.															*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*		12/09/2009 (MarkT) : created									*/
/*		2/25/2010 (AaronO) : changed power-on message					*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#include	"stdtypes.h"
#include	"CerebotNano.h"
#include	"PmodRF1.h"
#include	"bit_bang_spi.h"
#include	"util.h"
#include	"config.h"


/* ------------------------------------------------------------ */
/*				Local Type Definitions							*/
/* ------------------------------------------------------------ */

//Defines reception timeout before throwing an error
#define WATCHDOG 364 //115 * 261.12ms = ~ 95.05s. (period of apprx. 3 expected transmissions)

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */

volatile WORD wTimer2Overflows;
volatile BYTE fTimeoutError;

//PmodCLS instructions
char szClearScreen[] = { 0x1B, '[', 'j', 0};
char szSetLineWrap[] = { 0x1B, '[', '0', 'h', 0};
char szGoToLine1[] = { 0x1B, '[', '0', ';', '0', 'H', 0};
char szGoToLine2[] = { 0x1B, '[', '1', ';', '0', 'H', 0};
char szBacklightOn[] = { 0x1B, '[','3', 'e', 0 };
char szWriteCursorOff[] = { 0x1B, '[', '0', 'n', 0 };
char szWriteEn[] = { 0x1B, '[', 'w', 0 };
char szCursorOff[] = { 0x1B, '[', '0', 'c', 0 };
char szEraseLine[] = { 0x1B, '[', '0', 'K', 0 };

/* ------------------------------------------------------------ */
/*				Local Variables									*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Macros											*/
/* ------------------------------------------------------------ */

#define		__startTimer2()		wTimer2Overflows = 0;\
								TCNT2 = 0;\
								TCCR2B |= (1 << CS22)|(1 << CS21)|(1 << CS20);\
								TIMSK2 |= (1 << TOIE2)
								
#define		__stopTimer2()		TCCR2B &= ~(1 << CS22)|(1 << CS21)|(1 << CS20);\
								TIMSK2 &= ~(1 << TOIE2)


/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */

void	DeviceInit (void);
void	AppInit(void);
void	Wait_ms(WORD ms);
void 	PrintSPI_string(char string[]);
void	PrintSPI_array(BYTE *prgbIn, BYTE bLen);


//Functions located in PmodTMPdriver.c 
extern void		InitPmodTMP(void);
extern int16_t	GetTemperature(void);

/* ------------------------------------------------------------ */
/*				Interrupt Service Routines						*/
/* ------------------------------------------------------------ */

//Data transmission watchdog interrupt
SIGNAL(SIG_OVERFLOW2){

	wTimer2Overflows += 1;
	if(wTimer2Overflows > WATCHDOG) {
		fTimeoutError = fTrue;
	}
}

/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */

/***	main
**
**	Synopsis:
**		st = main()
**
**	Parameters:
**		none
**
**	Return Values:
**		does not return
**
**	Errors:
**		none
**
**	Description:
**		Main program module. Performs basic board initialization
**		and then enters the main program loop.
*/

int main(void) {
	
	char buf[16 + 1];
    BYTE rgbPkt[16 + 1];
	BYTE fRxDone;
	BYTE cbPkt;


	DeviceInit();
	AppInit();


	Wait_ms(100);
	PrintSPI_string("Digilent PmodRF1"); //make sure cursor is homed on screen
	Wait_ms(5);
	PrintSPI_string(szGoToLine2);
	Wait_ms(5);
	PrintSPI_string("Weather Station");
	Wait_ms(5000);
	
	
	//This is the main program loop
	while(fTrue) {
	
		/**** The test fixture PmodRF1 should get ready to receive the packet **/
		// Place PmodRF1 transceiver in the RX_ON state.
		RF1WriteReg(rfrTrxState, rfstRxOn );
		__startTimer2(); //start watchdog timer

		/**** The PmodRF1 should be receiving a packet **/
		/*	at any time.  Wait here for either a packet reception, or a time out ***/
		fRxDone = fFalse; //init
		fTimeoutError = fFalse; //init

		//Wait for either a timeout or the IRQ pin to indicate RX
		while ( ( ! fRxDone) && ( ! fTimeoutError) ) {

			cli();			

			if(pinRF1IRQ & ( 1 << bnRF1IRQ )) { //RX done
				__stopTimer2();
				fRxDone = fTrue;
				// Place the BUT in the TRX_OFF state to save power.
				RF1WriteReg(rfrTrxState, rfstTrxOff);
				// Read the PmodRF1 interrupt status register.
				BRF1ReadReg(rfrIrqStatus);
			} //end if(pinBUT...

			else if(fTimeoutError) {
				__stopTimer2();
				// Place the BUT in the TRX_OFF state to save power.
				RF1WriteReg(rfrTrxState, rfstTrxOff);
				// Read the PmodRF1 interrupt status register.
				BRF1ReadReg(rfrIrqStatus);
			} //end else if...

			sei();		
		} //end while (( ! fRxD...
		
		/************************************************************/

		if ( fRxDone ) { //We received data from the partner PmodRF1	
			// Read data packet from PmodRF1 frame buffer.
			cbPkt = BRF1ReadPkt(0, rgbPkt);
			
			//Print contents of packet to lower line of PmodCLS
			PrintSPI_string(szGoToLine2);
			Wait_ms(2);
			PrintSPI_string(szEraseLine); //clear line
			Wait_ms(2);
			PrintSPI_array(rgbPkt, cbPkt);
		}
		else { // fTimeoutError = 1 //Print death message to screen
			PrintSPI_string(szGoToLine2);
			Wait_ms(2);
			PrintSPI_string(szEraseLine); //clear line
			Wait_ms(2);
			PrintSPI_string("Comm Link Down");
		}

		/************* Print the base temperature *****************/
		
		sprintf(buf, "Base (F): %3d", GetTemperature());
		PrintSPI_string(szGoToLine1);
		Wait_ms(2);
		PrintSPI_string(szEraseLine); //clear line
		Wait_ms(2);
		PrintSPI_string(buf);

		/************************************************************/

	}  //end while
} //end main()

/* ------------------------------------------------------------ */
/***	DeviceInit
**
**	Synopsis:
**		DeviceInit()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Initializes on chip peripheral devices to the default
**		state.
*/

void DeviceInit() {

	/* Turn all pins as outputs tied to ground.
	*/
	DDRB = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFF;

	PORTB = 0x00;
	PORTC = 0x00;
	PORTD = 0x00;

	//PmodTMP, pin3 is unused
	ddrJB9 &= ~(1 << bnJB9);
	prtJB9 |= (1 << bnJB9);
	InitPmodTMP(); //initialize the PmodTMP
	
   // Initialize the SPI interfaces.
	BitBangSpiInit();

	//Init additional signals for PmodRF1
	ddrRF1IRQ &= ~(1 << bnRF1IRQ); //input
	prtRF1IRQ &= ~(1 << bnRF1IRQ); //IRQ pin	
	prtRF1RST |= (1 << bnRF1RST);
	ddrRF1RST |= (1 << bnRF1RST); //RST pin	
	prtRF1SLPTR &= ~(1 << bnRF1SLPTR); //SLP_TR pin
	ddrRF1SLPTR &= ~(1 << bnRF1SLPTR);	
	prtRF1CLKM &= ~(1 << bnRF1CLKM); //CLKM pin floating
	ddrRF1CLKM &= ~(1 << bnRF1CLKM);
	
	//PmodCLS pins
	prtSpi &= ~((1 << bnSpiMosi)|(1 << bnSpiSck)); //LLV
	prtSpi |= (1 << bnSpiSS); //hold SS line high
	ddrSpi |= (1 << bnSpiSS)|(1 << bnSpiMosi)|(1 << bnSpiSck); //output

} //end DeviceInit()

/* ------------------------------------------------------------ */
/***	AppInit
**
**	Synopsis:
**		AppInit()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Performs application specific initialization. Sets devices
**		and global variables for application.
*/

void AppInit() {

	BYTE rfrTemp;


	//cut power to un-used peripherals
	PRR = (1 << PRTWI)|(1 << PRTIM0)|(1 << PRTIM1)|(1 << PRUSART0);

	//init globals
	wTimer2Overflows = 0;
	fTimeoutError = fFalse;
	

	/************ Initialize the PmodRF1 ****************/
	RF1Init();

   // Set the channel (center frequency) used for wireless communication.
    rfrTemp = BRF1ReadReg(rfrPhyCcCca);
    rfrTemp &= ~(mskChan);
    rfrTemp |= ( mskChan & rfchan ); // rfchan is defined in config.h
    RF1WriteReg(rfrPhyCcCca, rfrTemp);

    // Set the data rate for the PmodRF1 to 250kbps.
    RF1WriteReg(rfrTrxCtrl2, 0x0c);

    // Clear the PmodRF1 interrupt status register.
    BRF1ReadReg(rfrIrqStatus);

    // Enable PmodRF1 interrupt mask mode (polling).
    rfrTemp = BRF1ReadReg(rfrTrxCtrl1);
    rfrTemp |= ( 1 << bnIrqMaskMode );
    RF1WriteReg(rfrTrxCtrl1, rfrTemp);

    // Enable the TRX_END interrupt on the PmodRF1.
    RF1WriteReg(rfrIrqMask, ( 1 << bnMaskTrxEnd ));
	/******************************************************/

	//set up SPI peripheral
	/* Enable SPI, Master, set clock rate fck/64 =~62.5k bps */
	SPCR = (1 << SPE)|(1 << MSTR)|(1 << SPR0);
	
	//set up PmodCLS
	Wait_ms(100);
	PrintSPI_string(szSetLineWrap);
	Wait_ms(3);
	PrintSPI_string(szBacklightOn);
	Wait_ms(3);
	PrintSPI_string(szCursorOff);
	Wait_ms(3);
	PrintSPI_string(szClearScreen);
	Wait_ms(3);

	sei();  //enable global interrupts	
}

/* ------------------------------------------------------------ */
/***	Wait_ms
**
**	Synopsis:
**		Wait_ms(WORD)
**
**	Parameters:
**		WORD (range from 0 to 65535)
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Will wait for specified number of milliseconds.  Using a 
**		word variable allows for delays up to 65.535 seconds.  The value
**		in the for statement may need to be altered depending on how your
**		compiler translates this code into AVR assembly.  In assembly, it is
**		possible to generate exact delay values by counting the number of clock
**		cycles your loop takes to execute.  When writing code in C, the compiler
**		interprets the code, and translates it into assembly.  This process is 
**		notoriously inefficient and may vary between different versions of AVR Studio
**		and WinAVR GCC.  A handy method of calibrating the delay loop is to write a 
**		short program that toggles an LED on and off once per second using this 
**		function and using a watch to time how long it is actually taking to
**		complete. 
**
**		This code supports a clock of 1MHz (49).  The number in the for loop should be
**		~390 for 8MHz clock source.
**
*/

void Wait_ms(WORD delay) {

	WORD i;

	while(delay > 0){

		for( i = 0; i < 49; i ++){
			;;
		}
		delay -= 1;
	}
}

/* ------------------------------------------------------------ */
/***	PrintSPI_string
**
**	Synopsis:
**		PrintSPI_string()
**
**	Parameters:
**		character array, AKA: a string
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This routine sends a string, exactly as it comes in from caller
**		through the SPI.
*/

void PrintSPI_string(char string[]) {
	
	BYTE i;
	BYTE bJunk;
	i = 0;

	prtSpi &= ~(1 << bnSpiSS); //lower SS pin

	while (string[i]) {

		SPDR = string[i];
		/* Wait for transmission to complete */
		while(!(SPSR & (1 << SPIF)) ) {
			asm volatile("nop");
		}
		bJunk = SPDR; //get trash and do next byte
	
		i += 1;
	}

	prtSpi |= (1 << bnSpiSS); //raise SS pin
}

/* ------------------------------------------------------------ */
/***	PrintSPI_array
**
**	Synopsis:
**		PrintSPI_array()
**
**	Parameters:
**		BYTE *prgbIn: pointer to input array
**		BYTE bLen	: number of bytes to write
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This routine sends a BYTE, exactly as it comes in from caller
**		through the SPI.
*/

void PrintSPI_array(BYTE *prgbIn, BYTE bLen) {

	BYTE bJunk;

	prtSpi &= ~(1 << bnSpiSS); //lower SS pin

	while (bLen > 0) {

		SPDR = *prgbIn;
		/* Wait for transmission to complete */
		while(!(SPSR & (1 << SPIF)) ) {
			asm volatile("nop");
		}
		bJunk = SPDR; //get trash and do next byte
	
		prgbIn++; //advance pointer
		bLen -= 1; //decrement
	}

	prtSpi |= (1 << bnSpiSS); //raise SS pin
}