/************************************************************************/
/*																		*/
/*	PmodRF1.c	--  PmodRF1 Wireless Transceiver Interface              */
/*																		*/
/************************************************************************/
/*	Author: Michael T. Alexander										*/
/*	Copyright 2009, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*																		*/
/*  This module contains declarations and definitions for interfacing   */
/*  the wireless transceiver used on the PmodRF1.                       */
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*  05/22/2009 (MichaelA): created                                      */
/*  05/29/2009 (MichaelA): fixed bug in BRF1ReadPkt that resulted in an */
/*      incorrect LQI value being read                                  */
/*	11/02/2009 (MarkT): added support for multiple PmodRF1's for PmodRF1*/
/*						BT.												*/
/*	12/09/2009 (MarkT): removed support for multiple PmodRF1's for the 	*/
/*						reference design.  Only the bit-banged SPI		*/
/*						is supported now.								*/
/*																		*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */

#include <avr/io.h>
#include <avr/interrupt.h>
#include "config.h"
#include "PmodRF1.h"
#include "bit_bang_spi.h"
#include "util.h"

/* ------------------------------------------------------------ */
/*				Local Type Definitions							*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Local Variables									*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */

extern void	Wait_ms( WORD );

/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */
/***	RF1Init
**
**	Parameters:
**		none
**
**	Return Value:
**		register content
**
**	Errors:
**		none
**
**	Description:
**		Initializes the PmodRF1. All pins are set to their default
**      states and the PmodRF1 is placed into the TRX_OFF state.
**
**  NOTE: CLKM output is disabled to save power
**
*/
void    RF1Init( void )
{
    BYTE rfstCur;

	// Reset the PmodRF1.
	prtRF1RST &= ~( 1 << bnRF1RST );
	Wait_ms(1);
	prtRF1RST |= ( 1 << bnRF1RST );
	Wait_ms(1);
	
    // Get the current state of the PmodRF1.
    rfstCur = BRF1ReadReg(rfrTrxStatus);
    rfstCur &= mskTrxStatus;

    // Transition to the TRX_OFF state if we aren't already in it.
    if ( rfstPOn == rfstCur ) {
        
        // Send the command to transition to the TRX_OFF state.
        RF1WriteReg(rfrTrxState, rfstTrxOff);
        Wait_ms(1);

        // Wait for the transition to the TRX_OFF state to complete.
        rfstCur = BRF1ReadReg(rfrTrxStatus);
        rfstCur &= mskTrxStatus;
        while ( rfstTrxOff != rfstCur ) {
            rfstCur = BRF1ReadReg(rfrTrxStatus);
            rfstCur &= mskTrxStatus;
        }
    }

    // Disable CLKM output by the PmodRF1.
    RF1WriteReg(rfrTrxCtrl0, 0);

    // Initialize RF1 globals.
    rfrLqi = 0;
}

/* ------------------------------------------------------------ */
/***	BRF1ReadReg
**
**	Parameters:
**		rfr - address of the register to read
**
**	Return Value:
**		register content
**
**	Errors:
**		none
**
**	Description:
**		Read the value of a specified PmodRF1 register.
**
*/

BYTE    BRF1ReadReg( BYTE rfr )
{
    BYTE    rfrTemp;

    // Ensure a minimal of 0.5 us between SS high to low.
    DelayUs05();

	// Enable the PmodRF1 by driving SS low.
	prt_bb_ss &= ~( 1 << bn_bb_ss );
	
	// Transmit command and register address.
	BitBangSpiPutByte(rfcRegRead | rfr);  // discard PHY_STATUS for now

	// Transmit dummy data and receive register value.
	rfrTemp = BitBangSpiPutByte(0);
	
	prt_bb_ss |= ( 1 << bn_bb_ss );
	
    return rfrTemp;
}


/* ------------------------------------------------------------ */
/***	RF1WriteReg
**
**	Parameters:
**		rfr  - address of the register to write
**      bVal - register value to write
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Write the contents of a specified PmodRF1 register.
**
*/

void    RF1WriteReg( BYTE rfr, BYTE bVal )
{
    // Ensure a minimal of 0.5 us between SS high to low.
    DelayUs05();

	// Enable the PmodRF1 by driving SS low.
	prt_bb_ss &= ~( 1 << bn_bb_ss );

	// Transmit write register command and register address.
	BitBangSpiPutByte(rfcRegWrite | rfr ); // discard PHY_STATUS for now

	// Transmit register value and receive dummy data.
	BitBangSpiPutByte(bVal);

	prt_bb_ss |= ( 1 << bn_bb_ss );
}


/* ------------------------------------------------------------ */
/***	BRF1ReadFrame
**
**	Parameters:
**		pbBuff - pointer to a buffer to hold the frame payload
**
**	Return Value:
**		number of bytes contained in the frame payload
**
**	Errors:
**		none
**
**	Description:
**		Read the contents of the PmodRF1 frame buffer.
**
*/

BYTE    BRF1ReadFrame( BYTE* pbBuff )
{
    BYTE cbBuff;

    // Ensure a minimal of 0.5 us between SS high to low.
    DelayUs05();


   // Enable the PmodRF1 by driving SS low.
	prt_bb_ss &= ~( 1 << bn_bb_ss );

	// Transmit the read frame command to the PmodRF1.
	BitBangSpiPutByte(rfcFrameRead); // dicard PHY_STATUS byte for now

	// Transmit dummy data and receive the frame payload length.
	cbBuff = BitBangSpiPutByte(0);

	if ( cbBuff ) {
		// Transmit the frame payload.
		BitBangSpiGetBuff(0, pbBuff, cbBuff);

		// Transmit LQI.
		BitBangSpiPutByte(0);

		// Transmit ED.
		BitBangSpiPutByte(0);

		// Transmit RX_STATUS.
		BitBangSpiPutByte(0); // consider verifying RX_CRC_VALID is set
	}
	// Disable the PmodRF1 by driving SS high.
	prt_bb_ss |= ( 1 << bn_bb_ss );	

 
    return cbBuff;
}


/* ------------------------------------------------------------ */
/***	RF1WriteFrame
**
**	Parameters:
**		pbBuff - pointer to a buffer containing the frame payload
**      cbBuff - number of bytes in frame payload buffer
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Write a frame to the PmodRF1 frame buffer.
**
*/

void    RF1WriteFrame( BYTE* pbBuff, BYTE cbBuff )
{
    // Ensure a minimal of 0.5 us between SS high to low.
    DelayUs05();


	// Enable the PmodRF1 by driving SS low.
	prt_bb_ss &= ~( 1 << bn_bb_ss );

	// Transmit the write frame command to the PmodRF1.
	BitBangSpiPutByte(rfcFrameWrite); // discard PHY_STATUS byte for now

	// Transmit the frame length byte to the PmodRF1 frame buffer.
	BitBangSpiPutByte(cbBuff);

	// Transmit the frame payload to the PmodRF1 frame buffer.
	BitBangSpiPutBuff(pbBuff, cbBuff);

	// Disable the PmodRF1 by driving SS high.
	prt_bb_ss |= ( 1 << bn_bb_ss );	
}

/* ------------------------------------------------------------ */
/***	BRF1ReadPkt
**
**	Parameters:
**		prfadrSrc - pointer to a RF1 address variable to hold
**                  the source address of the packet received
**      pbPkt     - pointer to a BYTE buffer to hold the received
**                  packet
**
**	Return Value:
**		count of bytes contained in the packet
**
**	Errors:
**		none
**
**	Description:
**		Read a packet from the RF1 frame buffer. Note that
**      a count of 0 will be returned if the destination address
**      of the packet does not match the address (rfadrMac) of this
**      device. Upon sucecssful reception of a packet the link
**      quality indicator variable (rfrLqi) is updated.
**
**  Note: the source address does not have to be obtain, calling
**        this routine with 0 as the parameter for prfadrSrc
**        discards the source address of the received packet.
*/

BYTE    BRF1ReadPkt( BYTE* prfadrSrc, BYTE* pbPkt )
{
    BYTE    cbFrame;
    BYTE    cbPkt;
    BYTE    rfadrTemp;
    BYTE    rfrTemp;

    cbPkt = 0;

    // Ensure a minimal of 0.5 us between SS high to low.
    DelayUs05();


	// Enable the PmodRF1 by driving SS low.
	prt_bb_ss &= ~( 1 << bn_bb_ss );

	// Transmit the read frame command to the PmodRF1.
	BitBangSpiPutByte(rfcFrameRead); // dicard PHY_STATUS byte for now

	// Transmit dummy data and receive the frame payload length.
	cbFrame = BitBangSpiPutByte(0);

	if ( 4 <= cbFrame ) {
		// Frame should contain at least the source and destination
		// address and checksum.

		// Transmit dummy data and receive the source address.
		rfadrTemp = BitBangSpiPutByte(0);
		if ( 0 != prfadrSrc ) {
			*prfadrSrc = rfadrTemp;
		}

		// Transmit dummy data and receive the destination address.
		rfadrTemp = BitBangSpiPutByte(0);

		// Verify that this device is the target device.
		if (( rfadrRF1 == rfadrTemp ) || ( rfadrBcast == rfadrTemp )) {
			
			cbPkt = cbFrame - 4;

			// Receive the packet payload.
			BitBangSpiGetBuff(0, pbPkt, cbPkt);

			// Receive the lower byte of the frame checksum.
			BitBangSpiPutByte(0);

			// Receive the upper byte of the frame checksum.
			BitBangSpiPutByte(0);

			// Receive LQI.
			rfrLqi = BitBangSpiPutByte(0);

			// Receive ED.
			BitBangSpiPutByte(0);

			// Receive RX_STATUS.
			rfrTemp = BitBangSpiPutByte(0);

			// Verify that the CRC is valid.
			if ( ! ( rfrTemp & ( 1 << bnRxCrcValid ) ) ) {
				cbPkt = 0; // drop pkt
			}
		}
	}

	// Disable the PmodRF1 by driving SS high.
	prt_bb_ss |= ( 1 << bn_bb_ss );	
	
    return cbPkt;
}

/* ------------------------------------------------------------ */
/***	RF1WritePkt
**
**	Parameters:
**		rfadrDest - destination address
**      pbPkt     - pointer to a BYTE buffer containing the packet
**                  to write to the frame buffer
**      cbPkt     - count of bytes in packet to write
**
**	Return Value:
**		count of bytes contained in the packet
**
**	Errors:
**		none
**
**	Description:
**		Write a packet to RF1 frame buffer for transmission.
**
*/

void    RF1WritePkt( BYTE rfadrDest, BYTE* pbPkt, BYTE cbPkt )
{
    BYTE    cbFrame;

    // Ensure a minimal of 0.5 us between SS high to low.
    DelayUs05();


	// Enable the PmodRF1 by driving SS low.
	prt_bb_ss &= ~( 1 << bn_bb_ss );

	// Transmit the write frame command to the PmodRF1.
	BitBangSpiPutByte(rfcFrameWrite); // discard PHY_STATUS byte for now

	// Transmit the frame length to the PmodRF1 frame buffer.
	cbFrame = 4 + cbPkt;    // 2 bytes for addressing, 2 for FCS
	BitBangSpiPutByte(cbFrame);

	// Transmit the source address.
	BitBangSpiPutByte(rfadrRF1);

	// Transmit the destination address.
	BitBangSpiPutByte(rfadrDest);

	// Transmit the packet payload.
	BitBangSpiPutBuff(pbPkt, cbPkt);

	// Transmit dummy FCS byte 1.
	BitBangSpiPutByte(0);

	// Transmit dummy FCS byte 2.
	BitBangSpiPutByte(0);

	// Disable the PmodRF1 by driving SS high.
	prt_bb_ss |= ( 1 << bn_bb_ss );
}

/* ------------------------------------------------------------ */
/***	RF1SetMac
**
**	Parameters:
**		rfadrSet - MAC address of this device
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Set the MAC address of this device.
**
*/

void    RF1SetMac( BYTE rfadrSet )
{
    rfadrMac = rfadrSet;
}

/************************************************************************/
