/************************************************************************/
/*																		*/
/*	bit_bang_spi.c	--  simulated SPI Interface functions				*/
/*																		*/
/************************************************************************/
/*	Author: Mark Taylor													*/
/*	Copyright 2009, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*																		*/
/*  This module contains interface declarations for simulated SPI 		*/
/*	communication.														*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*  10/22/2009 (MarkT): created from Mike A.'s PmodRF1 SPI driver      	*/
/*																		*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */

#include "bit_bang_spi.h"

/* ------------------------------------------------------------ */
/*				Local Type Definitions							*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Local Variables									*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */
/***	BitBangSpiInit
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Initialize the SPI interface for use.
**
*/
void BitBangSpiInit(void) {

    // Configure SS, MOSI, MISO, and SCK pins.
	prt_bb_ss |= (1 << bn_bb_ss); //SS
	ddr_bb_ss |= (1 << bn_bb_ss);
	
	prt_bb_mosi &= ~(1 << bn_bb_mosi); //MOSI
	ddr_bb_mosi |= (1 << bn_bb_mosi);
	
	ddr_bb_miso &= ~(1 << bn_bb_miso); //MISO
	prt_bb_miso |= (1 << bn_bb_miso);
	
	prt_bb_sck |= (1 << bn_bb_sck); //SCK
	ddr_bb_sck |= (1 << bn_bb_sck);
}

/* ------------------------------------------------------------ */
/***	BitBangSpiEnable
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Enable SPI slave device.
*/
void BitBangSpiEnable() {

    prt_bb_ss &= ~( 1 << bn_bb_ss);
}


/* ------------------------------------------------------------ */
/***	BitBangSpiDisable
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Disable SPI slave device.
*/
void BitBangSpiDisable() {

    prt_bb_ss |= ( 1 << bn_bb_ss);
}

/* ------------------------------------------------------------ */
/***	BitBangSpiPutByte
**
**	Parameters:
**		bSnd - byte to send
**
**	Return Value:
**		byte received from slave
**
**	Errors:
**		none
**
**	Description:
**		Send a byte to a SPI slave and receive a byte from the
**      slave.
*/

BYTE BitBangSpiPutByte( BYTE bSnd ) {

	BYTE i;
	BYTE bResponse;
	
	bResponse = 0; //init
	

/*	Start clocking out StartConvertT command*/
	for(i = 0; i < 8; i ++) {
		//lower clock
		prt_bb_sck &= ~(1 << bn_bb_sck); //falling edge of clock
		
		//place data on MOSI on falling edge of clock (MSb first)
		if(bSnd & (1 << (7 - i))) {
			prt_bb_mosi |= (1 << bn_bb_mosi); //set dq if data bit set
		}
		else {
			prt_bb_mosi &= ~(1 << bn_bb_mosi); //otherwise clear dq
		}
		prt_bb_sck |= (1 << bn_bb_sck); //rising edge of clock
		
		//Sample data on the rising edge of clock
		if(pin_bb_miso & (1 << bn_bb_miso)) {
			bResponse |= (1 << (7-i)); //set bit
		}
	}

    return bResponse;
}

/* ------------------------------------------------------------ */
/***	BitBangSpiPutBuff
**
**	Parameters:
**		pbBuff - pointer to a buffer of bytes to send
**      cbBuff - number of bytes in buffer
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Send a buffer of bytes to an SPI slave. Discard any 
**      data received from the slave.
*/

void BitBangSpiPutBuff( BYTE* pbBuff, WORD cbBuff )
{
    while ( 0 < cbBuff ) {
        BitBangSpiPutByte(*pbBuff);
        pbBuff++;
        cbBuff--;
    }
}

/* ------------------------------------------------------------ */
/***	BitBangSpiGetBuff
**
**	Parameters:
**      bFill  - a filler byte sent to the slave
**		pbBuff - pointer to a buffer of bytes to receive
**      cbBuff - number of bytes to receive
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Receive a specified number of bytes from SPI slave.
**      The bFill byte is shifted to the slave on every transmission.
*/

void BitBangSpiGetBuff( BYTE bFill, BYTE* pbBuff, WORD cbBuff )
{
    while ( 0 < cbBuff ) {
        *pbBuff = BitBangSpiPutByte(bFill);
        pbBuff++;
        cbBuff--;
    }
}

/* ------------------------------------------------------------ */
/***	BitBangSpiPutGetBuff
**
**	Parameters:
**      pbSnd  - a buffer of bytes to send
**		pbRcv  - a buffer of bytes to receive slave data
**      cbSnd  - number of bytes to be transferred
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Transfer a specified number of bytes between master and
**      slave.
*/

void BitBangSpiPutGetBuff( BYTE* pbSnd, BYTE* pbRcv, WORD cbSnd )
{
    while ( 0 < cbSnd ) {
        *pbRcv = BitBangSpiPutByte(*pbSnd);
        pbRcv++;
        pbSnd++;
        cbSnd--;
    }
}

/************************************************************************/
